/**
 * JASMINe
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eventswitch.endpoints;

import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.ow2.jasmine.monitoring.eventswitch.messages.JasmineEvent;

/**
 * Dispatches received MBeanCmd messages on all outbound routes.
 */
public class MBeanCmdDispatcher {
    /**
     * Gets the version number for a given MBeanCmd
     * JASMINe Event Switch message header.
     *
     * @param header  First line of the MBeanCmd message.
     * @param format  Format to look for.
     *
     * @return  Version number.
     *
     * @throws UnsupportedOperationException  If header is invalid.
     */
    private static int getVersion(final String header, final String format) throws UnsupportedOperationException {
        try {
            if(header.indexOf(format)==0) {
                return Integer.parseInt(header.substring(format.length()));
            } else if(header.indexOf("time;date;server;domain;mbean;")==0) {
                return 0;
            }
        } catch(Exception e) {
            // Nothing
        }

        // If we came here no one returned
        throw new UnsupportedOperationException("Header not recognized: "+header);
    }

    /**
     * Reformats and dispatches a message received from MBeanCmd.
     *
     * Reformatting is done the following way:
     *
     *      - First, msg is split at each new line (to separate the header
     *        from the body) and then each line is split at each semicolon
     *        (to separate each field).
     *      - Afterwards, the message header and body are parsed in order to
     *        create the corresponding {@link JasmineEvent}.
     *
     * @param msg  Message received from MBeanCmd.
     *
     * @return  JasmineEvent to dispatch.
     */
    public JasmineEvent receivedMBeanCmdMessage(final String msg) {
        // Transform msg into String arrays for easier reading
        String[] message = ((String) msg).split("\n");
        int version = getVersion(message[0], "mbeancmd_values_v");
        if(version < 0 || version > 1) {
            throw new UnsupportedOperationException("Unknown MBeanCmd message version: "+version);
        }
        String[] msgHeader = message[1].split(";");
        String[] msgBody = message[2].split(";");

        // Separate header fields from data fields
        Map<String, Integer> header = new TreeMap<String, Integer>();
        List<String> probes = new LinkedList<String>();
        for(int i=0; i<msgHeader.length; i++) {
            if(msgHeader[i].length() > 0) {
                header.put(msgHeader[i], i);
                if(!nonProbeFields.contains(";"+msgHeader[i]+";")) {
                    probes.add(msgHeader[i]);
                }
            }
        }

        // Fill the JasmineEvent based on parsed header + message body
        JasmineEvent event = new JasmineEvent();
        Date timestamp = new Date(Long.parseLong(msgBody[header.get("time")]));
        event.setDomain(msgBody[header.get("domain")]);
        event.setServer(msgBody[header.get("server")]);
        String mbean = msgBody[header.get("mbean")]+":";
        String sname = msgBody[header.get("sname")];
        for(String s: probes) {
            event.addEvent("mbeancmd", mbean+s, msgBody[header.get(s)], timestamp,sname);
        }

        // This will make Mule dispatch the event
        // on all outbound routes for this endpoint
        return event;
    }

    /**
     * Calls {@link MBeanCmdDispatcher#receivedMBeanCmdMessage(String)}. This
     * method is required since some Mule connectors will serialize into bytes:
     * http://docs.huihoo.com/soa/mule/1.3/Hello%20World%20and%20friends.html
     *
     * @param msg  Message received from MBeanCmd.
     *
     * @return  JasmineEvent to dispatch.
     */
    public JasmineEvent receivedMBeanCmdMessage(final byte[] msg) {
        return receivedMBeanCmdMessage(new String(msg));
    }

    /**
     * List of fields sent by MBeanCmd that are not probes (for example, date
     * or server name), each element must be surrounded by a ";".
     *
     * Any field that is not in this list will be treated as probe and will be
     * added to the {@link JasmineEvent#getEvents()} list.
     */
    private static final String nonProbeFields = ";date;time;server;domain;mbean;name;sname;";
}
