/**
 * JASMINe
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eventswitch.endpoints;

import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.mule.umo.lifecycle.Startable;
import org.mule.umo.lifecycle.Stoppable;
import org.ow2.jasmine.monitoring.mbeancmd.CommandDispatcher;

/**
 * A wrapper class for MBeanCmd. It works the following way:
 *
 *     1. Waits for the configuration to be set. The configuration may launch
 *        one or more MBeanCmd instances.
 *     2. Creates instances of MBeanCmd with the given configuration.
 *     3. MBeanCmd is then expected to connect to an instance of
 *        {@link MBeanCmdDispatcher} via MuleClient.
 */
public class MBeanCmdWrapper implements Startable, Stoppable {
    /**
     * MBeanCmd configuration.
     *
     * @see MBeanCmdWrapper#setConfiguration(String)
     */
    private String configuration = null;

    /**
     * Delay to start MBeanCmd.
     */
    private long delay = 0;

    /**
     * List of all wrapped MBeanCmd instances.
     */
    private List<Thread> wrappedInstances = new LinkedList<Thread>();

    /**
     * Transforms a command line command into an array of Strings. This is
     * functionally equivalent to the transformation Java does when giving the
     * command line arguments to your main(String[]).
     *
     * @param line  Command line to transform
     * @return      String[] that corresponds to the command line.
     */
    protected static synchronized String[] parseCommandLine(final String line) {
        int lastQuoteEnd = 0;
        List<String> args = new LinkedList<String>();

        // Spacing characters: space, tab
        Pattern space = Pattern.compile("\\ |\\\t");

        // Special form: strings enclosed in quotes. Those may contain
        // everything (including spacing characters) but quotes.
        Matcher matcher = Pattern.compile("\\\"[^\\\"]*\\\"").matcher(line);

        // Find all elements enclosed in quotes
        while(matcher.find()) {
            // Remember where the current quote starts and ends
            int start = matcher.start();
            int end = matcher.end();

            // Split at each spacing character between the end of the last
            // quote (initially, the beginning of the string) and the
            // beginning of the one we're handling right now.
            for(String c: space.split(line.substring(lastQuoteEnd, start))) {
                if(c.length()>0) {
                    args.add(c);
                }
            }

            // Don't split the string in quotes and accept quoted empty strings
            args.add(line.substring(start + 1, end - 1));

            // Now the end of the last quote is the end of the current quote
            lastQuoteEnd = end;
        }

        // Split at each spacing character after the end of the last quote
        for(String c: space.split(line.substring(lastQuoteEnd))) {
            if(c.length()>0) {
                args.add(c);
            }
        }

        // Return as a String array
        return args.toArray(new String[args.size()]);
    }

    /**
     * Sets the configuration to use in the MBeanCmd.
     *
     * @param configuration  MBeanCmd configuration, accepts exactly the same
     *                       options as mbean.jar used as a command line tool.
     *                       Remember to have the -jasmine option.
     */
    public void setConfiguration(final String configuration) {
        this.configuration = configuration;
    }

    /**
     * Set the delay to start MBeanCmd.
     * @param delay delay to start MBeanCmd
     */
    public void setDelay(final String delay) {
        this.delay = Long.parseLong(delay);
    }

    /**
     * Creates MBeanCmd threads for each MBeanCmd specified as configuration.
     *
     * @see org.mule.umo.lifecycle.Startable#start()
     * @see MBeanCmdWrapper#setConfiguration(String)
     */
    public void start() {
        synchronized (wrappedInstances) {
            // Each new line marks another MBeanCmd to launch
            for(final String c: configuration.split("\\\r|\\\n")) {
                final String[] args = parseCommandLine(c);
                if(args.length>0) {
                    // Verify that command contains the -jasmine option and a URI
                    boolean jasmineOption = false;
                    boolean jasmineItem = false;
                    for(String arg:args) {
                        if(jasmineOption) {
                            if(arg.length()>0) {
                                jasmineItem = true;
                            }
                            break;
                        } else if(arg.equals("-jasmine")) {
                            jasmineOption = true;
                        }
                    }
                    if(!jasmineOption || !jasmineItem) {
                        throw new IllegalArgumentException("-jasmine option not specified for the MBeanCmd container");
                    }

                    // Start the command dispatcher's start method (never returns)
                    Thread t = new Thread() {
                        public void run() {
                            final long time = System.currentTimeMillis();
                            synchronized (this) {
                                while(System.currentTimeMillis() < time + delay) {
                                    try {
                                        wait(delay);
                                    } catch (InterruptedException e) {
                                        Thread.currentThread().interrupt();
                                    }
                                }
                            }
                            CommandDispatcher.start(args);
                        }
                    };
                    wrappedInstances.add(t);
                    t.start();
                }
            }
        }
    }

    /**
     * Stops all wrapped MBeanCmd instances.
     *
     * @see org.mule.umo.lifecycle.Stoppable#stop()
     */
    public void stop() {
        synchronized (wrappedInstances) {
            for(Thread t: wrappedInstances) {
                t.interrupt();
            }
            wrappedInstances.clear();
        }
    }
}
